%undefine _missing_build_ids_terminate_build
%bcond_without openmpi

%define openmpiversion  4.1.6
%define openmpiflavor gcc
%define openmpinversion %( echo %{openmpiversion} | tr -d . )

%if %{with openmpi}
%define destdir /usr/local/%{rname}/openmpi-%{openmpiversion}/%{version}
%define openmpiprereq prereq openmpi
%else
%define destdir /usr/local/%{rname}/%{version}
%define openmpiprereq %{nil}
%endif

# type: string (root path to install modulefiles)
%{!?modulefile_path: %define modulefile_path /usr/local/share/Modules/modulefiles/%{name}}
%if %{with openmpi}
# type: string (subdir to install modulefile)
%{!?modulefile_subdir: %define modulefile_subdir openmpi-%{openmpiversion}}
%else
%{!?modulefile_subdir: %define modulefile_subdir %{nil}}
%endif
# type: string (name of modulefile)
%{!?modulefile_name: %define modulefile_name %{version}}

%define meson meson132
%define dyninstversion 12.3.0
%define dyninstnversion %( echo %{dyninstversion} | tr -d . )
%define boostversion 1.76.0
%define boostnversion 176

%define rname hpctoolkit
Version:	2024.01.1
%define		nversion %( echo %{version} | tr -d . )
%if %{with openmpi}
Name:		%{rname}%{nversion}-openmpi%{openmpinversion}
%else
Name:		%{rname}%{nversion}
%endif
Release:	1%{?dist}
Summary:	Integrated suite of tools for measurement and analysis of program performance

Group:		Development/Libraries
License:	BSD
URL:		http://hpctoolkit.org/
Source0:	https://gitlab.com/hpctoolkit/hpctoolkit/-/archive/%{version}/hpctoolkit-%{version}.tar.bz2
Patch1:		fixes.patch
# because it builds its own versions of some of these libraries we do not want it to provide some of this stuff
AutoReqProv:	0

# From https://gitlab.com/hpctoolkit/hpctoolkit
# firs tcommon
BuildRequires:	git gcc gcc-c++ ccache ninja-build cmake pkg-config bzip2-devel elfutils-devel xerces-c-devel binutils-devel yaml-cpp-devel libpfm-devel xxhash-devel zlib-devel dyninst%{dyninstnversion}-boost%{boostnversion}-devel libunwind-devel
BuildRequires:	libmonitor-devel intel-xed-devel m4 libtool autoconf automake
Requires:	libmonitor intel-xed
BuildRequires:	%{meson}
%if %{rhel} < 9
BuildRequires:	python39 python39-pip python39-rpm-macros dyninst%{dyninstnversion}-boost%{boostnversion}-devel
Requires:	python39 dyninst%{dyninstnversion}-boost%{boostnversion}
%else
BuildRequires:	python3 pipx boost-devel
%endif
#BuildRequires:	libxml2-devel papi-devel
%if %{with openmpi}
BuildRequires:  openmpi-%{openmpiflavor}-devel = %{openmpiversion}
Requires:       openmpi-%{openmpiflavor}-runtime >= %{openmpiversion}
%endif
Requires:       environment-modules-local
Requires:	libxml2 papi libstdc++

%description
HPCToolkit is an integrated suite of tools for measurement and
analysis of program performance on computers ranging from multicore
desktop systems to GPU-accelerated supercomputers. HPCToolkit
provides accurate measurements of a program's work, resource
consumption, and inefficiency, correlates these metrics with the
program's source code, works with multilingual, fully optimized
binaries, has very low measurement overhead, and scales to large
parallel systems. HPCToolkit's measurements provide support for
analyzing a program execution cost, inefficiency, and scaling
characteristics both within and across nodes of a parallel system.

%package doc
Summary:	HPCToolkit documentation
Group:		Development/Libraries
Requires:	%{name} = %{version}
AutoReqProv:	0

%description doc
HPCToolkit documentation

%package devel
Summary:	Development libraries and include files for HPCToolkit
Group:		Development/Libraries
Requires:	%{name} = %{version}
Requires:	papi-devel libxml2-devel
%if %{with openmpi}
Requires:	openmpi-%{openmpiflavor}-devel = %{openmpiversion}
%endif
AutoReqProv:	0

%description devel
Development libraries and include files for HPCToolkit

%prep
%setup -q -n %{rname}-%{version}
%patch1 -p1 -b .fixes

%build
%if %{with openmpi}
# configure openmpi
module load openmpi
%endif
%if %{rhel} < 9
module load dyninst
%else
export BOOST_ROOT=/usr
export DYNINSTHOME=/usr
export TBBHOME=/usr
%endif
cat > local_properties <<ENDCONFIG
#[binaries]
#python = /usr/bin/python3.9

[properties]
prefix_bzip = '/usr'
prefix_tbb = '$TBBHOME'
prefix_lzma = '/usr'
prefix_zlib = '/usr'
prefix_xerces = '/usr'
prefix_elfutils = '/usr'
prefix_libunwind = '/usr'
prefix_yaml_cpp = '/usr'
prefix_dyninst = '$DYNINSTHOME'
prefix_libmonitor = '/usr'
prefix_libiberty = '/usr'
prefix_xed = '/usr'
prefix_perfmon = '/usr'
prefix_boost = '$BOOST_ROOT'
#  'papi',
#  'opencl',
#  'gtpin',
#  'igc',
#  'level0',
#  'rocm_hip',
#  'rocm_hsa',
#  'rocm_tracer',
#  'rocm_profiler',
#  'cuda',
ENDCONFIG

#../configure MPICXX=mpiCC MPICC=mpicc MPIF77=mpif77 --prefix=%{destdir} \
#     --with-externals=$TEMPDEST --with-papi=/usr
%{meson} setup builddir/  --native-file ./local_properties --prefix %{destdir}
cd builddir/
%{meson} compile

%install
rm -rf %{buildroot}
%if %{with openmpi}
# configure openmpi
module load openmpi
%endif
%if %{rhel} < 9
module load dyninst
%else
export BOOST_ROOT=/usr
%endif
cd builddir
DESTDIR=%{buildroot} %{meson} install

# Fix man page suffix
rename .1hpctoolkit .1 %{buildroot}%{destdir}/share/man/man1/*

mkdir -p $RPM_BUILD_ROOT/%{modulefile_path}/%{modulefile_subdir}
cat <<EOF >$RPM_BUILD_ROOT/%{modulefile_path}/%{modulefile_subdir}/%{version}
#%Module

proc ModulesHelp { } {
   puts stderr "This module adds HPCToolkit %{version} to various paths"
}

module-whatis   "Sets up HPCToolkit %{version} in your environment"

%{openmpiprereq}
prepend-path PATH "%{destdir}/bin"
setenv HPCTOOLKIT "%{destdir}"
prepend-path MANPATH "%{destdir}/share/man"
EOF

cd %{buildroot}%{destdir}
sed -i 's|/usr/bin/env python|%{__python3}|' libexec/hpctoolkit/hpcsummary
rm -f lib/hpctoolkit/lib*.la

%check
%if %{with openmpi}
module load openmpi
%endif
%if %{rhel} < 9
module load dyninst
%endif
cd builddir
%{meson} test

%clean
rm -rf %{buildroot}


%files
%defattr(-,root,root,-)
%dir /usr/local/%{rname}
%if %{with openmpi}
%dir /usr/local/%{rname}/openmpi*
%endif
%dir %{destdir}
%{destdir}/bin
%dir %{destdir}/lib
%dir %{destdir}/lib/hpctoolkit
%{destdir}/lib/hpctoolkit/hash-file
%{destdir}/lib/hpctoolkit/lib*.so*
%{destdir}/libexec
%dir %{destdir}/share
%{destdir}/share/man
%dir /usr/local/share/Modules/modulefiles/%{name}
%dir %{modulefile_path}
%dir %{modulefile_path}/%{modulefile_subdir}
%{modulefile_path}/%{modulefile_subdir}/%{version}
%{destdir}/share/doc

%files devel
%{destdir}/include
%{destdir}/lib/hpctoolkit/lib*.a

%files doc
%{destdir}/share/doc
%{destdir}/share/hpctoolkit

%changelog
* Sun Oct 06 2024 Josko Plazonic <plazonic@princeton.edu> - 2024.01.1-1
- update to current version

* Tue Sep 24 2013 Josko Plazonic <plazonic@math.princeton.edu>
- initial build
